package LedgerSMB::Project;

# First draft,  backport to 1.2, then will change to use
# base qw(LedgerSMB::DBObject) --CT
use LedgerSMB::Form;
use strict;

sub __validate__ {
}

sub new {
    my $class = shift @_;
    my %args  = (ref($_[0]) eq 'HASH')? %{$_[0]}: @_;
    my $base  = $args{base};
    my $mode  = $args{copy};
    my $self  = bless {}, $class;
    my @mergelist;
    if ( defined $args{merge} ){
        @mergelist = @{ $args{merge} };
    } elsif (defined $mode && ( $mode eq 'list')) {
        $self->error('Mergelist not set');
    }
    else {
        @mergelist = ();
    }
    if ( !$base->isa('Form') ) {
        $self->error("Constructor called without Form object arg");
    }

    my $attr;
    if (lc($mode) eq 'base'){
        $self->merge($base, keys => ['dbh', '_roles', '_user', '_locale', 
			'_request']);
    }
    elsif (lc($mode) eq 'list'){
        $self->merge($base, keys => ['dbh', '_roles', '_user', '_locale', 
			'_request']);
        $self->merge($base, keys => \@mergelist);
    }
    else {
        $self->merge($base);
    }
    $self->__validate__();
    $self->{_order_method} = {};
    return $self;
}

sub call_procedure {
    my $self     = shift @_;
    my %args     = @_;
    my $procname = $args{procname};
    my $schema   = $args{schema};
    my @call_args;
    @call_args = @{ $args{args} } if defined $args{args};
    my $order_by = $args{order_by};
    my $query_rc;
    my $argstr   = "";
    my @results;

    if (!defined $procname){
        $self->error('Undefined function in call_procedure.');
    }

    $procname = $self->{dbh}->quote_identifier($procname);
    
     $schema = $schema || $LedgerSMB::Sysconfig::db_namespace;
    
    if ($schema){
         $schema = $self->{dbh}->quote_identifier($schema);
         $procname = "$schema.$procname";
    }
    for ( 1 .. scalar @call_args ) {
        $argstr .= "?, ";
    }
    $argstr =~ s/\, $//;
    my $query = "SELECT * FROM $procname()";
    if ($order_by){
        $query .= " ORDER BY $order_by";
    }
    $query =~ s/\(\)/($argstr)/;
    my $sth = $self->{dbh}->prepare($query);
    if (scalar @call_args){
        $query_rc = $sth->execute(@call_args);
        if (!$query_rc and !$args{continue_on_error}){
              $self->dberror($self->{dbh}->errstr . ": " . $query);
        }
    } else {
        $query_rc = $sth->execute();
        if (!$query_rc and !$args{continue_on_error}){
              $self->dberror($self->{dbh}->errstr . ':' . $query);
        }
    }
    my @types = @{$sth->{TYPE}};
    my @names = @{$sth->{NAME_lc}};
    while ( my $ref = $sth->fetchrow_hashref('NAME_lc') ) {
        for (0 .. $#names){
            #   numeric            float4/real
             if ($types[$_] == 3 or $types[$_] == 2) {
                   $ref->{$names[$_]} = Math::BigFloat->new($ref->{$names[$_]});
                }
         }
       push @results, $ref;
     }
     return @results;
}
sub merge {
    my ( $self, $src ) = @_;
    for my $arg ( $self, $src ) {
        shift;
    }
    my %args  = @_;
    my @keys;
    if (defined $args{keys}){
         @keys  = @{ $args{keys} };
    }
    my $index = $args{index};
    if ( !scalar @keys ) {
        @keys = keys %{$src};
    }
    for my $arg ( @keys ) {
        my $dst_arg;
        if ($index) {
            $dst_arg = $arg . "_$index";
        }
        else {
            $dst_arg = $arg;
        }
        $self->{$dst_arg} = $src->{$arg};
    }
}

sub exec_method {
    my $self   = shift @_;
    my %args     = @_;
    my $funcname = $args{funcname};

    my $schema   = $args{schema} || $LedgerSMB::Sysconfig::db_namespace;

    my @in_args;
    @in_args = @{ $args{args} } if $args{args};

    my @call_args;

    my $query = "
        SELECT proname, pronargs, proargnames FROM pg_proc 
         WHERE proname = ? 
               AND pronamespace = 
               coalesce((SELECT oid FROM pg_namespace WHERE nspname = ?), 
                        pronamespace)
    ";
    my $sth   = $self->{dbh}->prepare(
                $query
    );
    $sth->execute($funcname, $schema) 
        || $self->error($DBI::errstr . "in exec_method");
    my $ref;

    $ref = $sth->fetchrow_hashref('NAME_lc');
    
    my $pargs = $ref->{proargnames};
    my @proc_args;

    if ( !$ref->{proname} ) {    # no such function
        # If the function doesn't exist, $funcname gets zeroed?
        $self->error( "No such function:  $funcname");
       die;
  }
      $ref->{pronargs} = 0 unless defined $ref->{pronargs};
          # If the user provided args..
        for (@in_args) { push @call_args, $_ } ;
        $self->{call_args} = \@call_args;
        return $self->call_procedure( procname => $funcname, args => \@call_args, order_by => $self->{_order_method}->{"$funcname"}, schema=>$schema);
}

sub error {
    use LedgerSMB::Form;
    shift @_;
    Form->error(@_);
}
# END BACKPORTING SECTION

sub get_invoice_report {
    my ($self) = @_;
    @{$self->{inv_report_lines}} = $self->exec_method(funcname => 'project__get_invoices', args=>[$self->{project_id}]);
    return @{$self->{inv_report_lines}};
}

sub get_by_id {
   my ($self, $id) = @_;
   if (!defined $id) { 
      $id = $self->{id} || $self->{project_id};
   }
   my $project;
   ($project) = $self->exec_method(funcname => 'project__get_by_id', 
                                   args => [$id]);
   $self->merge($project);   
}

1 
